% does cross sectional dividend yields

clear all
close all
addpath('data');

st = 20; % 20 start in 46, avoid 0 dividends
en = 1; % how many to chop from end in 2009 -- 1 avoids big 2009 outlier
sizebm = 0; % use size and bm portfolios
bmonly = 1; % use bm portfolios
sizeonly = 0; % use size portfolios 
rho = 0.96;
k = -(1-rho)*log(1-rho)-rho*log(rho);

if sizebm
    x = load('ff_25_returns.txt');
    r = 1+x(:,2:end)/100;

    x = load('ff_25_returns_nodiv.txt');
    cal = x(:,1);
    rx = 1+x(:,2:end)/100;

    lbls = {'s g '; 's v2'; 's v3'; 's v4'; 's v ';...
        's2g '; 's2v2'; 's2v3'; 's2v4'; 's2v ';...
        's3g '; 's3v2'; 's3v3'; 's3v4'; 's3v ';...
        's4g '; 's4v2'; 's4v3'; 's4v4'; 's4v ';...
        'l g '; 'l v2'; 'l v3'; 'l v4'; 'l v '};
end;

if bmonly
    x = load('ff_bm_returns.txt');
%    r = 1+x(:,6:10)/100; % this choice uses quintiles. There are also deciles
    r = 1+x(:,11:20)/100; % this choice uses deciles
% note there is a huge outlier in 2009 coming from what looks like a big
% dividend payment in march 2009. real? makes any difference?
    x = load('ff_bm_returns_nodivs.txt');
    cal = x(:,1);
%    rx = 1+x(:,6:10)/100; % quintiles
    rx = 1+x(:,11:20)/100; % deciles

%    lbls = {'g'; '2'; '3'; '4'; 'v'};
   lbls = {'growth'; '2'; '3'; '4'; '5'; '6'; '7'; '8'; '9'; 'value'};
end;

if sizeonly;
    x = load('fama_french_size_portfolios.txt');
%    r = 1+x(:,6:10)/100; % this choice uses quintiles. There are also deciles
    r = 1+x(:,11:20)/100; % this choice uses deciles
% note there is a huge outlier in 2009 coming from what looks like a big
% dividend payment in march 2009. real? makes any difference?
    x = load('fama_french_size_portfolios_nodivs.txt');
    cal = x(:,1);
%    rx = 1+x(:,6:10)/100; % quintiles
    rx = 1+x(:,11:20)/100; % deciles

%    lbls = {'g'; '2'; '3'; '4'; 'v'};
   lbls = {'small'; '2'; '3'; '4'; '5'; '6'; '7'; '8'; '9'; 'large'};
end;


x = load('vwr.txt');
vwr = 1+x(3:end,2);
vwrx = 1+x(3:end,3);
vwdp = vwr./vwrx-1;
vwdd = [NaN; vwdp(2:end,:)./vwdp(1:end-1,:).*vwrx(2:end,:)];
vwddi = [NaN; exp(log(vwr(2:end,:))-k+rho*(log(vwdp(2:end))-log(vwdp(1:end-1))))];

N = size(r,2); 

%((P1+D1)/P0*(P0/P1) -1)=D1/P1 This is DP at the end of the year
dp = r./rx-1; 

%(D1/P1)/(D0/P0)*P1/P0=D1/D0
dd = [NaN*ones(1,N); dp(2:end,:)./dp(1:end-1,:).*rx(2:end,:)];
ddimp = [ NaN*ones(1,N);  exp( log(r(2:end,:))-k+rho*log(dp(2:end,:))-log(dp(1:end-1,:)))];


cal = cal(st:end-en);
r = r(st:end-en,:);
rx = rx(st:end-en,:);
dp =  dp(st:end-en,:);
dd = dd(st:end-en,:);
ddimp = ddimp(st:end-en,:);

vwr = vwr(st:end-en,:);
vwrx = vwrx(st:end-en,:);
vwdp =  vwdp(st:end-en,:);
vwdd = vwdd(st:end-en,:);
vwddi = vwddi(st:end-en,:);

if any(dp==0); 
    'some dp are zero, will bomb';
end

T = size(dp,1);
N = size(r,2);

figure; 
plot(cal,100*dp); 
hold on
plot(cal, 100*vwdp,'-k','linewidth',2)
title('Portfolio dividend yields')

figure
plot(cal,100*dd);
hold on
plot(cal, 100*vwdd,'-k','linewidth',2)

k = 10; 
vwrlr = log(vwr(1:T-k+1));
rlr = log(r(1:T-k+1,:));
vwdlr = log(vwdd(1:T-k+1));
dlr = log(dd(1:T-k+1,:));
for i = 2:k; 
    vwrlr = vwrlr+rho^(i-1)*log(vwr(i:T-k+i));
    rlr = rlr+rho^(i-1)*log(r(i:T-k+i,:));
    vwdlr = vwdlr+rho^(i-1)*log(vwdd(i:T-k+i));
    dlr = dlr+rho^(i-1)*log(dd(i:T-k+i,:));
end;

figure;
plot(cal(1:T-k+1),vwrlr,'-k','linewidth',2);
hold on
plot(cal(1:T-k+1),rlr);
title('long run return')

x = [ones(T-k+1,1) vwrlr];
y = rlr; 
[bv,sebv,R2v,R2vadj,v,F] = olsgmm(y,x,0,0);
figure;
plot((1:10)',mean(rlr),'-v');
hold on
plot((1:10)',mean(vwrlr)*bv(2,:)','-or');
legend('E(r)','\beta');
title('long run capm');

figure;
plot(cal(1:T-k+1),vwdlr,'-k','linewidth',2);
hold on
plot(cal(1:T-k+1),dlr);
title('long run dividend growth');

x = [ones(T-k+1,1) vwdlr];
y = dlr; 
[bv,sebv,R2v,R2vadj,v,F] = olsgmm(y,x,0,0);
figure;
plot((1:10)',mean(dlr),'-v');
hold on
plot((1:10)',bv(2,:)','-or');
legend('E(d)','\beta');
title('long run capm -- cashflow betas');



disp('regressing individual returns and dd on their own dp');
fprintf(' %8.8s %8.8s %8.8s %8.8s  \n','port', 'b','t','R2')
for i = 1:N;
    x = [ones(T-1,1) log(dp(1:T-1,i))];
    y = [log(r(2:T,i)) log(dd(2:T,i)) log(ddimp(2:T,i)) log(dp(2:T,i)) ];
    [bv,sebv,R2v,R2vadj,v,F] = olsgmm(y,x,0,0);
    fprintf('  r %8.8s %8.3f %8.3f %8.3f \n', char(lbls(i)),bv(2,1),bv(2,1)/sebv(2,1),R2v(1));
    fprintf(' dd %8.8s %8.3f %8.3f %8.3f \n', char(lbls(i)),bv(2,2),bv(2,2)/sebv(2,2),R2v(2));
    fprintf('ddi %8.8s %8.3f %8.3f %8.3f \n', char(lbls(i)),bv(2,3),bv(2,3)/sebv(2,3),R2v(3));
    fprintf(' dp %8.8s %8.3f %8.3f %8.3f \n', char(lbls(i)),bv(2,4),bv(2,4)/sebv(2,4),R2v(4));
    disp(' ');
end;

% portfolio averages
if sizebm
    disp(' '); 
    disp('Portfolio averages')
    disp('labels')
    disp(reshape(lbls,5,5)')
    disp('return');
    disp(reshape(100*mean(log(r)),5,5)');
    disp('dd');
    disp(reshape(100*mean(log(dd)),5,5)');
    disp('e^mean log dp')
    disp(reshape(100*(exp(mean(log(dp)))),5,5)');
    disp('mean  dp')
    disp(reshape(100*mean(dp),5,5)');    
    disp('(1-rho) mean log dp')
    disp(reshape(100*(1-0.96)*mean(log(dp)),5,5)');    
    
    
    figure;
    subplot(2,2,1)
    bar3(reshape(100*mean(log(r)),5,5)');
    xlabel('value')
    ylabel('size')
    title('Return');
    subplot(2,2,2);
    bar3(reshape(100*mean(log(dd)),5,5)');
    title('Dividend growth');
    subplot(2,2,3);
    bar3(reshape(100*(exp(mean(log(dp)))),5,5)');
    title('Dividend yield');
end;

if (bmonly | sizeonly)
disp(' '); 
    disp('Portfolio averages')
    disp('labels')
    disp(lbls')
    disp('return');
    disp(100*mean(log(r)));
    disp('dd');
    disp(100*mean(log(dd)));
    disp('mean  dd implied')
    disp(100*mean(log(ddimp)));
    disp('e^mean log dp')
    disp(100*(exp(mean(log(dp)))));
    disp('mean  dp')
    disp(100*mean(dp));
    disp('(1-rho)mean log dp')
    disp(100*(1-0.96)*mean(log(dp)));
    
    
    figure
    plot((1:N)',100*mean(log(r)),'-v','linewidth',2);
    hold on
    plot((1:N)',100*mean(log(ddimp)),'--ro','linewidth',2);
    plot((1:N)',100*mean(log(dd)),'-ro','linewidth',2);    
 %   plot((1:N)',100*exp(mean(log(dp))),'--g*','linewidth',2);
%    plot((1:N)',100*mean(dp)+100*mean(log(dd)),'--v','linewidth',2)
    plot((1:N)',100*(mean(dp)),'-g*','linewidth',2);
     
    legend('Return','\Deltad, identity','\Deltad, direct','D/P',2);
    xlabel('B/M portfolio number')
    if bmonly
       set(gca,'xticklabel',{'growth', '2', '3', '4', '5', '6', '7', '8', '9', 'value'})
        print -depsc2 ff_bm_deciles_rdp.eps
    end
    if sizeonly
       set(gca,'xticklabel',{'small', '2', '3', '4', '5', '6', '7', '8', '9', 'large'})
       print -depsc2 ff_size_deciles_rdp.eps
    end 
   
end;


% pure cross section

disp('pure cross section')

y = mean(log(r(2:T,:)))'; 
x = [ones(N,1) mean(log(dp(1:T-1,:)))' ]; 
[bv,sebv,R2v,R2vadj,v,F] = olsgmm(y,x,0,0);

fprintf('regression of mean log return on mean log dp\n'); 
fprintf('       %8.8s %8.8s %8.8s \n','const','dp','b/(1-rho)'); 
fprintf('    r   %8.3f %8.3f %8.3f \n',bv,bv(2)/(1-rho)); 
bv1 = bv;

y = mean(log(dd(2:T,:)))'; 
x = [ones(N,1) mean(log(dp(1:T-1,:)))' ]; 
[bv,sebv,R2v,R2vadj,v,F] = olsgmm(y,x,0,0);
fprintf('   Dd   %8.3f %8.3f %8.3f \n',bv,bv(2)/(1-rho)); 
bv2 = bv;

y = mean(log(ddimp(2:T,:)))'; 
x = [ones(N,1) mean(log(dp(1:T-1,:)))' ]; 
[bv,sebv,R2v,R2vadj,v,F] = olsgmm(y,x,0,0);
fprintf('  Ddi   %8.3f %8.3f %8.3f \n',bv,bv(2)/(1-rho)); 
bv3 = bv;

y = rho/T*(log(dp(T,:))-log(dp(1,:)))'; 
x = [ones(N,1) mean(log(dp(1:T-1,:)))' ]; 
[bv,sebv,R2v,R2vadj,v,F] = olsgmm(y,x,0,0);
fprintf('dp term %8.3f %8.3f %8.3f \n',bv,bv(2)/(1-rho));
bv4 = bv;

fprintf('sums\n')
fprintf('r, ddimp, dpterm %8.3f \n', (bv1(2)-bv3(2)+bv4(2))/(1-rho) );


% pooled time-series cross section forecast


fprintf('Pooled forecasting regression  \n');

x = [ones((T-1)*N,1) reshape(log(dp(1:T-1,:)),(T-1)*N,1)]; 
y = reshape(log(dp(2:T,:)),(T-1)*N,1); 
[bv,sebv,R2v,R2vadj,v,F] = olsgmm(y,x,0,0); 
fprintf('       %8.8s %8.8s %8.8s %8.8s \n','const','dp','b/(1-rho)','R2'); 
fprintf('   dp  %8.3f %8.3f %8.3s %8.3f  \n',bv,' ',R2v);
phi = bv(2);

y = reshape(log(r(2:T,:)),(T-1)*N,1); 
[bv,sebv,R2v,R2vadj,v,F] = olsgmm(y,x,0,0);
fprintf('    r  %8.3f %8.3f %8.3f %8.3f  \n',bv,bv(2)/(1-rho*phi),R2v); 

y = reshape(log(dd(2:T,:)),(T-1)*N,1); 
[bv,sebv,R2v,R2vadj,v,F] = olsgmm(y,x,0,0); 
fprintf('   dd  %8.3f %8.3f %8.3f %8.3f \n',bv,bv(2)/(1-rho*phi),R2v); 

y = reshape(log(ddimp(2:T,:)),(T-1)*N,1); 
[bv,sebv,R2v,R2vadj,v,F] = olsgmm(y,x,0,0); 
fprintf('  ddi  %8.3f %8.3f %8.3f %8.3f  \n',bv,bv(2)/(1-rho*phi),R2v); 

disp(' ');

% pooled time-series cross section forecast with time dummies
fprintf('forecasting regression with time dummies\n');

x = [ kron(ones(N,1), eye(T-1)) reshape(log(dp(1:T-1,:)),N*(T-1),1)];

y = reshape(log(dp(2:T,:)),(T-1)*N,1);
[bv,sebv,R2v,R2vadj,v,F] = olsgmm(y,x,0,0); 
fprintf('   dp  %8.3f %8.3f %8.3s %8.3f \n',bv(end-1:end),' ',R2v); 
phi = bv(end);
y = reshape(log(r(2:T,:)),(T-1)*N,1);
[bv,sebv,R2v,R2vadj,v,F] = olsgmm(y,x,0,0); 
fprintf('    r  %8.3f %8.3f %8.3f %8.3f \n',bv(end-1:end),bv(end)/(1-rho*phi),R2v); 
y = reshape(log(dd(2:T,:)),(T-1)*N,1);
[bv,sebv,R2v,R2vadj,v,F] = olsgmm(y,x,0,0); 
fprintf('   dd  %8.3f %8.3f %8.3f %8.3f \n',bv(end-1:end),bv(end)/(1-rho*phi),R2v); 
y = reshape(log(ddimp(2:T,:)),(T-1)*N,1);
[bv,sebv,R2v,R2vadj,v,F] = olsgmm(y,x,0,0); 
fprintf('   ddi %8.3f %8.3f %8.3f %8.3f \n',bv(end-1:end),bv(end)/(1-rho*phi),R2v); 


% check with taking out time-means
y = reshape(log(r(2:T,:)),(T-1)*N,1);
logdp3 = log(dp(1:T-1,:)) - mean(log(dp(1:T-1,:)),2)*ones(1,N);
x = [ ones((T-1)*N,1) reshape(logdp3,N*(T-1),1) ]; 
[bv,sebv,R2v,R2vadj,v,F] = olsgmm(y,x,0,0); 
fprintf('return regression taking out time means\n')
fprintf('    r  %8.3f %8.3f  \n',bv); 

% pooled time-series cross section forecast with portfolio dummies

x = [kron(eye(N), ones(T-1,1)) reshape(log(dp(1:T-1,:)),N*(T-1),1)]; 
fprintf('forecasting regression  with portfolio dummies\n');
fprintf('       %8.8s %8.8s %8.8s %8.8s \n','const','dp','b/(1-rho)','R2'); 

y = reshape(log(dp(2:T,:)),(T-1)*N,1);
[bv,sebv,R2v,R2vadj,v,F] = olsgmm(y,x,0,0); 
phi = bv(end);
fprintf('   dp %8.3f %8.3f %8.8s %8.3f \n',bv(end-1:end),' ',R2v); 
y = reshape(log(r(2:T,:)),(T-1)*N,1);
[bv,sebv,R2v,R2vadj,v,F] = olsgmm(y,x,0,0); 
fprintf('    r %8.3f %8.3f %8.3f %8.3f \n',bv(end-1:end),bv(end)/(1-rho*phi),R2v); 
y = reshape(log(dd(2:T,:)),(T-1)*N,1);
[bv,sebv,R2v,R2vadj,v,F] = olsgmm(y,x,0,0); 
fprintf('   dd %8.3f %8.3f %8.3f %8.3f \n',bv(end-1:end),bv(end)/(1-rho*phi),R2v); 
y = reshape(log(ddimp(2:T,:)),(T-1)*N,1);
[bv,sebv,R2v,R2vadj,v,F] = olsgmm(y,x,0,0); 
fprintf('  ddi %8.3f %8.3f %8.3f %8.3f \n',bv(end-1:end),bv(end)/(1-rho*phi),R2v); 

% check taking out portfolio means\
fprintf('return taking out portfolio means\n');
logdp3 = log(dp(1:T-1,:)) - ones(T-1,1)*mean(log(dp(1:T-1,:)));
x = [ones((T-1)*N,1) reshape(logdp3,N*(T-1),1)]; 
[bv,sebv,R2v,R2vadj,v,F] = olsgmm(y,x,0,0); 
fprintf('   r  %8.3f %8.3f  \n',bv); 

% new. dp and growth in dp since individual stocks cant use firm dummies


y = reshape(log(r(3:T,:)),(T-2)*N,1); 
x = [ones((T-2)*N,1) reshape(log(dp(2:T-1,:)),N*(T-2),1) ...
                      reshape(log(dp(2:T-1,:))-log(dp(1:T-2,:)),N*(T-2),1) ]; 
%                      reshape(log(dp(2:T-1,:))-log(dp(1:T-2,:)),N*(T-2),1) ]; 

[bv,sebv,R2v,R2vadj,v,F] = olsgmm(y,x,0,0); 
fprintf('       %8.8s %8.8s %8.8s %8.8s \n','const','dp-1', 'Ddp-1','r2'); 
fprintf('    r  %8.3f %8.3f %8.3f %8.3f \n',bv,R2v); 

y = reshape(log(dd(3:T,:)),(T-2)*N,1); 
[bv,sebv,R2v,R2vadj,v,F] = olsgmm(y,x,0,0); 
fprintf('    dd  %8.3f %8.3f %8.3f %8.3f \n',bv,R2v); 

y = reshape(log(ddimp(3:T,:)),(T-2)*N,1); 
[bv,sebv,R2v,R2vadj,v,F] = olsgmm(y,x,0,0); 
fprintf('   ddi  %8.3f %8.3f %8.3f %8.3f \n',bv,R2v); 

y = reshape(log(dp(3:T,:)),(T-2)*N,1); 
[bv,sebv,R2v,R2vadj,v,F] = olsgmm(y,x,0,0); 
fprintf('   dp  %8.3f %8.3f %8.3f %8.3f \n',bv,R2v); 

