% Discount Rates
% Main Program for Time-series Analysis

clear all;
close all;
redograph = 0; % protects printing of graphs that have manually added arrows, etc. 

addpath('data');   % I keep all my data in the same place, and then separate directories for each problem set; 
                      %  this lets matlab find data for load commands. Omit if you're keeping
                      %  data in the same directory as your program
addpath('functions');  % Again, I keep programs such as regress_jc all in the same place; omit if you keep them 
                         % in the current directory. 

Startdate = 19470101;
Enddate = 20100630;                         

%% Section 2.1.1 Simple DP Regression
' *******Simple DP Regressions ***********' 

% Monthly Returns
x = load('vwr_m.txt');   % Note: This is CRSP VW Portfolio of NYSE, AMEX and NASDAQ
vw_dates = x(:,1); 
R  = 1+x(:,2); 
Rx = 1+x(:,3);
tt = find(and(vw_dates>=Startdate,vw_dates<=Enddate));
R = R(tt);
Rx = Rx(tt);
[yr,mo,day,vw_dates_number] = decode_date(vw_dates(tt));

T = size(R,1); 
Ra = (R(1:T-11)).*(R(2:T-10)).*(R(3:T-9)).*(R(4:T-8)).*(R(5:T-7)).*(R(6:T-6)).*...
     (R(7:T-5)).*(R(8:T-4)).*(R(9:T-3)).*(R(10:T-2)).*(R(11:T-1)).*(R(12:T));

Rxa = (Rx(1:T-11)).*(Rx(2:T-10)).*(Rx(3:T-9)).*(Rx(4:T-8)).*(Rx(5:T-7)).*(Rx(6:T-6)).*...
      (Rx(7:T-5)).*(Rx(8:T-4)).*(Rx(9:T-3)).*(Rx(10:T-2)).*(Rx(11:T-1)).*(Rx(12:T));
DP = Ra./Rxa-1; %(P1+D1)/P0 * P0/P1 -1;   
DP = [ ones(11,1)*NaN; DP];

h2 = 7*12;
r7y = (R(1:end-h2+1)); 
for j = 1:h2-1; 
    r7y = r7y.*(R(1+j:end-h2+1+j)); 
end; 
r7y = [ones(h2-1,1)*NaN;r7y];

figure; 
plot(vw_dates_number,400*DP,'--b','linewidth',2); 
hold on
plot(vw_dates_number(1:end-h2),100*(r7y(1+h2:end).^(1/7)-1),'-r','linewidth',2); 
axis([datenum(1945,1,1),datenum(2010,6,1) -inf inf]);
datetick('x','keeplimits'); 
%title('4 x D/P and Annualized Following 7-Year Return');
legend('4 x D/P','Return') 
print -depsc2 dp_return.eps

x = load('sbbi_m.txt'); 
tb_dates = x(:,1); 
Rtb = 1+x(:,3); 
tt = find(and(tb_dates>=Startdate,tb_dates<=Enddate));
Rtb = Rtb(tt);

Re = R-Rtb;
T = size(R,1); 

lhv = Re(13:end);
rhv = [ones(T-12,1) DP(12:end-1)];
[bv,sebv,R2v,R2vadj,v,F] = olsgmm(lhv,rhv,0,0);
disp('Table 1 Simple DP regression with horizon k');
fprintf('             %6s %6s %6s %6s %6s \n','b','t(b)','R^2','s(ER)','s(ER)/ER');
fprintf('1 month      %6.1f %6.1f %6.2f %6.2f %6.2f \n',bv(2),bv(2)/sebv(2),R2v,std(bv(2)*DP(12:end-1))*100,std(bv(2)*DP(12:end-1))/mean(Re));

% Annual Returns
x = load('vwr.txt');   % Note: This is CRSP VW Portfolio of NYSE, AMEX and NASDAQ
vw_dates = x(:,1); 
R  = 1+x(:,2); 
Rx = 1+x(:,3);
DP = R./Rx-1; % (D_T+P_t)/(P_t-1) * P_t-1/P_t
DD = [NaN; DP(2:end)./DP(1:end-1).*Rx(2:end)];  % D_t/P_t / (D_t-1/P_t-1) *P_t/P_t
r = log(R);
dp = log(DP);
rho = 0.96;
dd = log(DD);
dd_id = [ NaN;  (r(2:end)+rho*dp(2:end)-dp(1:end-1))]; % NOTE: not real data! means identities should hold exactly

tt = find(and(vw_dates>=Startdate,vw_dates<=Enddate));
R = R(tt);
Rx = Rx(tt);
DP = DP(tt);
DD = DD(tt);
r = r(tt);
vw_dates = vw_dates(tt); 
dp = dp(tt);
dd = dd(tt);
dd_id = dd_id(tt);

x = load('sbbi.txt'); 
tb_dates = x(:,1); 
Rtb = 1+x(:,3); 
tt = find(and(tb_dates>=Startdate,tb_dates<=Enddate));
Rtb = Rtb(tt);

Re = R-Rtb;
T = size(R,1); 

lhvR = (R(2:end));
lhvRtb = (Rtb(2:end)); 
rhv = [ones(T-1,1) DP(1:end-1)];
for i = 1:5;
     lhvRx = lhvR-lhvRtb; 
     lags = i; % implements hansen-hodrick errors
     weight = 0; 
     [bvRx(:,i),sebvRx(:,i),R2vRx(:,i),R2vadj,v,F] = olsgmm(lhvRx,rhv,lags,weight);     % excess 
     lhvR = lhvR(1:end-1).*(R(2+i:end)); % cumulate returns
     lhvRtb = lhvRtb(1:end-1).*(Rtb(2+i:end)); 
     rhv = rhv(1:end-1,:); 
 end; 

fprintf('1 year       %6.1f %6.1f %6.2f %6.2f %6.2f \n',bvRx(2,1),bvRx(2,1)/sebvRx(2,1),R2vRx(1),std(bvRx(2,1)*DP(1:end-1))*100,std(bvRx(2,1)*DP(1:end-1))/mean(Re));
fprintf('5 year       %6.1f %6.1f %6.2f %6.2f %6.2f \n',bvRx(2,5),bvRx(2,5)/sebvRx(2,5),R2vRx(5),std(bvRx(2,5)*DP(1:end-1))*100,std(bvRx(2,5)*DP(1:end-1))/mean(lhvRx));

%% 2.1.2 Bubbles, volatility and regressions

' *************** Long Horizons, VAR with DP ***********'

% Long horizon VAR

r15y = r(1:end-14);
dd15y = dd(1:end-14);
for j = 1:14; 
    r15y = r15y+(rho^j)*r(1+j:end-14+j); 
    dd15y = dd15y+(rho^j)*dd(1+j:end-14+j);
end; 
r15y = [ones(14,1)*NaN; r15y];
dd15y = [ones(14,1)*NaN; dd15y];

lhv = [r15y(16:end) dd15y(16:end) dp(16:end)*rho^15];
rhv = [ones(T-15,1) dp(1:end-15)];
[bv,sebv,R2v,R2vadj,v,F] = olsgmm(lhv,rhv,15,0);

disp('Long-horizon return regression coefficients');
fprintf('             %6s %6s %6s \n','r','dd','dp');
fprintf('Direct k=15  %6.2f %6.2f %6.2f \n',bv(2,:));

% 1-year VAR

lhv = [r(2:end) dd(2:end) dp(2:end)];
rhv = [ones(T-1,1) dp(1:end-1)];
[bv,sebv,R2v,R2vadj,v,F] = olsgmm(lhv,rhv,0,0);
err = lhv - rhv*bv;
sbx = std(rhv*bv);
R2jc = sbx./mean(lhv);
A = [zeros(3,2) bv(2,:)'];

br = bv(2,1);
bd = bv(2,2);
phi = bv(2,3);
br15 = br*(1-(rho*phi)^15)/(1-rho*phi);
bd15 = bd*(1-(rho*phi)^15)/(1-rho*phi);
bdp15 = (rho*phi)^15;
brl = br/(1-rho*phi);
bdl = bd/(1-rho*phi);

fprintf('VAR k=15     %6.2f %6.2f %6.2f \n',br15,bd15,bdp15);
fprintf('VAR k=infty  %6.2f %6.2f %6.2f \n',brl,bdl,0);

disp('Regressions of log returns, dividend growth and dp on lagged dp');
fprintf('           %6s %6s %6s %6s %6s \n','b','t','R2','sbx%','s/E');
fprintf('r_t+1      %6.2f %6.2f %6.2f %6.2f %6.2f\n',br,br/sebv(2,1),R2v(1),sbx(1)*100,R2jc(1));
fprintf('dd_t+1     %6.2f %6.2f %6.2f %6.2f %6.2f \n',bd,bd/sebv(2,2),R2v(2),sbx(2)*100,R2jc(2));
fprintf('dp_t+1     %6.2f %6.2f %6.2f %6.2f %6.2f \n',phi,phi/sebv(2,3),R2v(3),sbx(3)*100,R2jc(3));


% variance analysis - to be used in Section 2.4 (Use period 1952-2009)

lhv = [r(7:end) dd(7:end) dp(7:end)];
rhv = [ones(T-6,1) dp(6:end-1)];
[bv,sebv,R2v,R2vadj,v,F] = olsgmm(lhv,rhv,0,0);
err = lhv - rhv*bv;
A = [zeros(3,2) bv(2,:)'];

% long run return forecast for later
lrErdp = bv(2,1)/(1-rho*bv(2,3))*dp(6:end); 
lrErdp = (lrErdp-mean(lrErdp))+mean(dp(6:end)); % used in plot of E(rlr below) 

varx = cov(err);
An = A; 
for i = 1:10; 
    varx = varx + An*varx*An';
    An = An*An;
end; % doubling algorithm computes very fast. 

lrvar = inv(eye(3)-rho*A)*A*varx*A'*(inv(eye(3)-rho*A))'; 
er = [1 0 0]'; 
ed = [0 1 0]'; 

fprintf('decomposition of variance with dp only, raw and in percent\n'); 
fprintf('                  %10.9s %10.9s %10.9s\n','var','sd','pct');
fprintf(' var(dp)          %10.3f %10.3f %10.3f\n',varx(3,3),varx(3,3)^0.5,varx(3,3)/varx(3,3)*100); 
fprintf(' var(Erlr)        %10.3f %10.3f %10.3f\n',er'*lrvar*er,(er'*lrvar*er)^0.5,er'*lrvar*er/varx(3,3)*100); 
fprintf(' var(Edlr)        %10.3f %10.3f %10.3f\n',ed'*lrvar*ed,(ed'*lrvar*ed)^0.5,ed'*lrvar*ed/varx(3,3)*100);
fprintf(' -2cov(Erlr,Edlr) %10.3f %10.3s %10.3f \n',-2*er'*lrvar*ed,' ',-2*er'*lrvar*ed/varx(3,3)*100); 
fprintf(' corr(Erlr,Edlr ) %10.3f  \n',er'*lrvar*ed/((er'*lrvar*er * ed'*lrvar*ed)^0.5)); 
fprintf(' sum              %10.3f %10.3f \n',er'*lrvar*er+ed'*lrvar*ed-2*er'*lrvar*ed,...
    (er'*lrvar*er+ed'*lrvar*ed-2*er'*lrvar*ed)/varx(3,3)*100); 

disp('note VAR produces different variances than sample averages.')
disp('table of standard deviations')
fprintf('        %10.9s  %10.9s  %10.9s  \n','r','dd','dp');
fprintf(' sample %10.3f  %10.3f  %10.3f  \n',std(r),std(dd(2:end)),std(dp) );
fprintf(' VAR    %10.3f  %10.3f  %10.3f  \n',diag(varx)'.^0.5);

%% 2.3A adding lags of DP, lags of DD


% 1-year VAR

lhv = [r(2:end) dd(2:end) dp(2:end)];
rhv = [ones(T-1,1) dp(1:end-1)];
[bv,sebv,R2v,R2vadj,v,F] = olsgmm(lhv,rhv,0,0);
err = lhv - rhv*bv;

br = bv(2,1);
bd = bv(2,2);
phi = bv(2,3);

disp('Regressions of log returns, dividend growth and dp on lagged dp');
fprintf('           %6s %6s %6s \n','b','t','R2');
fprintf('r_t+1      %6.2f (%6.2f) %6.2f \n',br,br/sebv(2,1),R2v(1));
fprintf('dd_t+1     %6.2f (%6.2f) %6.2f \n',bd,bd/sebv(2,2),R2v(2));
fprintf('dp_t+1     %6.2f (%6.2f) %6.2f \n',phi,phi/sebv(2,3),R2v(3));

lhv = [r(3:end) dd(3:end) dp(3:end) dp(3:end)-dp(2:end-1)];
rhv = [ones(T-2,1) dp(2:end-1) dp(2:end-1)-dp(1:end-2)];
[bv,sebv,R2v,R2vadj,v,F] = olsgmm(lhv,rhv,0,0);
err = lhv - rhv*bv;
br = bv(2:3,1);
bd = bv(2:3,2);
phi = bv(2:3,3);
phi2 = bv(2:3,4);
sigE = std(rhv*bv); 
R2jc = std(rhv*bv)./mean(lhv);

disp('Two lags of dp');
fprintf('           %6s %6s  %6s   %6s  %6s %6s %6s \n','b0','b1','t0','t1','R2','sigE%','R2JC');
fprintf('r_t+1      %6.2f %6.2f (%6.2f) (%6.2f) %6.2f %6.2f %6.2f \n',br,br./sebv(2:3,1),R2v(1),100*sigE(1),R2jc(1));
fprintf('dd_t+1     %6.2f %6.2f (%6.2f) (%6.2f) %6.2f %6.2f %6.2f \n',bd,bd./sebv(2:3,2),R2v(2),100*sigE(2),R2jc(2));
fprintf('dp_t+1     %6.2f %6.2f (%6.2f) (%6.2f) %6.2f %6.2f %6.2f \n',phi,phi./sebv(2:3,3),R2v(3),100*sigE(3),R2jc(3));
fprintf('D dp_t+1   %6.2f %6.2f (%6.2f) (%6.2f) %6.2f %6.2f %6.2f \n',phi2,phi2./sebv(2:3,4),R2v(4),100*sigE(4),R2jc(4));

lhv = [r(2:end) dd(2:end) dp(2:end)];
rhv = [ones(T-1,1) dp(1:end-1) dd(1:end-1)];
[bv,sebv,R2v,R2vadj,v,F] = olsgmm(lhv,rhv,0,0);
err = lhv - rhv*bv;
br = bv(2:3,1);
bd = bv(2:3,2);
phi = bv(2:3,3);

disp('dp and dd');
fprintf('           %6s %6s %6s %6s %6s \n','dp','dd','tdp','tdd','R2');
fprintf('r_t+1      %6.2f %6.2f (%6.2f) (%6.2f) %6.2f \n',br,br./sebv(2:3,1),R2v(1));
fprintf('dd_t+1     %6.2f %6.2f (%6.2f) (%6.2f) %6.2f \n',bd,bd./sebv(2:3,2),R2v(2));
fprintf('dp_t+1     %6.2f %6.2f (%6.2f) (%6.2f) %6.2f \n',phi,phi./sebv(2:3,3),R2v(3));


lhv = [r(3:end) dd(3:end) dp(3:end)];
rhv = [ones(T-2,1) dp(2:end-1) (dd(2:end-1)+dd(1:end-2))/2];
[bv,sebv,R2v,R2vadj,v,F] = olsgmm(lhv,rhv,0,0);
err = lhv - rhv*bv;
br = bv(2:3,1);
bd = bv(2:3,2);
phi = bv(2:3,3);

disp('dp and (dd1+dd2)/2');
fprintf('           %6s %6s %6s %6s %6s \n','dp','dd','tdp','tdd','R2');
fprintf('r_t+1      %6.2f %6.2f (%6.2f) (%6.2f) %6.2f \n',br,br./sebv(2:3,1),R2v(1));
fprintf('dd_t+1     %6.2f %6.2f (%6.2f) (%6.2f) %6.2f \n',bd,bd./sebv(2:3,2),R2v(2));
fprintf('dp_t+1     %6.2f %6.2f (%6.2f) (%6.2f) %6.2f \n',phi,phi./sebv(2:3,3),R2v(3));

lhv = [r(4:end) dd(4:end) dp(4:end)];
rhv = [ones(T-3,1) dp(3:end-1) (dd(3:end-1)+dd(2:end-2)+dd(1:end-3))/3];
[bv,sebv,R2v,R2vadj,v,F] = olsgmm(lhv,rhv,0,0);
err = lhv - rhv*bv;
br = bv(2:3,1);
bd = bv(2:3,2);
phi = bv(2:3,3);

disp('dp and (dd1+dd2+dd3)/3');
fprintf('           %6s %6s %6s %6s %6s \n','dp','dd','tdp','tdd','R2');
fprintf('r_t+1      %6.2f %6.2f (%6.2f) (%6.2f) %6.2f \n',br,br./sebv(2:3,1),R2v(1));
fprintf('dd_t+1     %6.2f %6.2f (%6.2f) (%6.2f) %6.2f \n',bd,bd./sebv(2:3,2),R2v(2));
fprintf('dp_t+1     %6.2f %6.2f (%6.2f) (%6.2f) %6.2f \n',phi,phi./sebv(2:3,3),R2v(3));


lhv = [r(5:end) dd(5:end) dp(5:end)];
rhv = [ones(T-4,1) dp(4:end-1) (dd(4:end-1)+dd(3:end-2)+dd(2:end-3)+dd(1:end-4))/4];
[bv,sebv,R2v,R2vadj,v,F] = olsgmm(lhv,rhv,0,0);
err = lhv - rhv*bv;
br = bv(2:3,1);
bd = bv(2:3,2);
phi = bv(2:3,3);

disp('dp and (dd1+dd2+dd3+dd4)/4');
fprintf('           %6s %6s %6s %6s %6s \n','dp','dd','tdp','tdd','R2');
fprintf('r_t+1      %6.2f %6.2f (%6.2f) (%6.2f) %6.2f \n',br,br./sebv(2:3,1),R2v(1));
fprintf('dd_t+1     %6.2f %6.2f (%6.2f) (%6.2f) %6.2f \n',bd,bd./sebv(2:3,2),R2v(2));
fprintf('dp_t+1     %6.2f %6.2f (%6.2f) (%6.2f) %6.2f \n',phi,phi./sebv(2:3,3),R2v(3));


%% 2.4 Cay

% Goyal-Welch's website
% x = load('PredictorData2009_a.txt');
% t = x(:,1);
% cay = x(:,10);   % Consumption-to-wealth ratio of Lettau-Ludvigson
% tt = find(and(t>=floor(Startdate/10000),t<=Enddate/10000));
% cay = cay(tt);

' ************************ CAY ***************'

% Sydney Ludvigson's website - 1952 to 2008
x = load('cay_2010.txt') ;
date_cay = x(:,1);
cayt = x(:,end); 
cay_a = cayt(4:4:size(cayt,1),1); % annual, Q4 observations,

cay_a = cay_a/std(cay_a); % renormalizing to unit standard deviation

disp(' renormalizingto sigma = 1'); 

date_cay_a = date_cay(4:4:size(date_cay,1),1); 
date_cay_a = floor(date_cay_a)/100; % now "2009" for 2009Q4

rhv = [ones(T-6,1) dp(6:end-1) cay_a(1:end-1)];
lhv = [r(7:end) dd(7:end) dd_id(7:end) dp(7:end) cay_a(2:end)]; 
[bv,sebv,R2v,R2vadj,v,F] = olsgmm(lhv,rhv,0,0);
fitted = rhv*bv; 
err = lhv - rhv*bv;
b = bv(2:3,:); 
t = b./sebv(2:3,:);
fit2 = [ones(T-5,1) dp(6:end) cay_a(1:end)]*bv; % include last data point

% plot of fitted value

rhvdp = [ones(T-6,1) dp(6:end-1)];
lhvdp = [r(7:end) ]; 
[bvdp,sebvdp,R2vdp,R2vadjdp,vdp,Fdp] = olsgmm(lhvdp,rhvdp,0,0);
fitted_dp = [ones(T-5,1) dp(6:end)]*bvdp; 

figure
plot((date_cay_a),100*[fit2(:,1) fitted_dp],'LineWidth',2); 
hold on
plot((date_cay_a(1:end-1)),r(7:end)*100,'-r'); 
%legend('dp and cay','dp only','Actual r_{t+1}',3);
text(1992,32,'dp and cay','color','b')
text(1987,-10,'dp only','color',[0 0.5 0]);
text(1960,32,'Actual return r_{t+1}','color','r');

axis([1950 2010 -20 40]);
if redograph
    print -depsc2 cay_1.eps
end

% print table 
fprintf('forecasts using dp and cay \n'); 
fprintf('         %6.9s   %6.9s \n', 'dp','cay' );
fprintf(' r_t+1   %6.3f   %6.3f \n', b(:,1)); 
fprintf('dd_t+1   %6.3f   %6.3f \n', b(:,2)); 
fprintf('ddid_t+1 %6.3f   %6.3f \n', b(:,3)); 
fprintf('dp_t+1   %6.3f   %6.3f \n', b(:,4));
fprintf('cay_t+1  %6.3f   %6.3f \n', b(:,5)); 
disp(' '); 

fprintf('t statistics using dp and cay \n'); 
fprintf('         %6.9s   %6.9s  \n', 'dp','cay' );
fprintf(' r_t+1  (%6.2f) (%6.2f) \n', t(:,1)); 
fprintf('dd_t+1  (%6.2f) (%6.2f) \n', t(:,2)); 
fprintf('ddid_t+1(%6.2f) (%6.2f) \n', t(:,3)); 
fprintf('dp_t+1  (%6.2f) (%6.2f) \n', t(:,4));
fprintf('cay_t+1 (%6.2f) (%6.2f) \n', t(:,5)); 
disp(' '); 

fprintf('Other statistics \n');
fprintf('         %6.9s   %6.9s %6.9s \n', 'R2','s(Ey)','s(Ey)/Ey' );
fprintf(' r_t+1   %6.2f   %6.2f %6.2f \n', R2v(1),std(fitted(:,1))*100,std(fitted(:,1))/mean(lhv(:,1))); 
fprintf('dd_t+1   %6.2f   %6.2f %6.2f \n', R2v(2),std(fitted(:,2))*100,std(fitted(:,2))/mean(lhv(:,2))); 
fprintf('ddid_t+1 %6.2f   %6.2f %6.2f \n', R2v(3),std(fitted(:,3))*100,std(fitted(:,3))/mean(lhv(:,3))); 
fprintf('dp_t+1   %6.2f   %6.2f %6.2f \n', R2v(4),std(fitted(:,4))*100,std(fitted(:,4))/mean(lhv(:,4)));
fprintf('cay_t+1  %6.2f   %6.2f %6.2f \n', R2v(5),std(fitted(:,5))*100,std(fitted(:,5))/mean(lhv(:,5))); 
disp(' '); 

A = [zeros(5,3) b'];    % expand to [r dd ddi dp cay]' = A []'_t  VAR 

% long-run coefficients, Sum rho^(j-1) A^j = A(I-rho*A)^(-1)
Al = A*inv(eye(5)-rho*A);
% variance of long run returns and dividend growth
fitlong = [dp(6:end) cay_a(1:end)]*Al(:,4:5)';
stlong = std(fitlong); 

fprintf('long run forecast coeffs using dp and cay, and sd lof long run  \n'); 
fprintf('         %6.9s   %6.9s   %6.9s \n', 'dp','cay','sigma');
fprintf('Sum rhor %6.2f   %6.3f   %6.3f \n', Al(1,4:5),stlong(1)); 
fprintf('Sum rhod %6.2f   %6.3f   %6.3f \n', Al(2,4:5),stlong(2));
fprintf('Sum rhdid%6.2f   %6.3f   %6.3f \n', Al(3,4:5),stlong(3));
disp('');

% plot of long-run forecasts

figure;
bcdp = Al(1,4:5);
Erlong = [dp(6:end) cay_a]*bcdp';
Erlong = (Erlong-mean(Erlong))+mean(dp(6:end));
plot(date_cay_a,[Erlong lrErdp],'linewidth',2);
hold on
plot(date_cay_a,[dp(6:end)],'--r','linewidth',2);
text(1998, -3.5, 'dp','color','r')
text(1980,-4.3,'Long-run r forecasts','color','b')
%legend('E(r^{lr}|dp,cay)','E(r^{lr}|dp)','dp');
if redograph
    print -depsc2 Erlr_cay.eps
end

h = 100; % horizon for IR 
dd_option = 1; % 1 for implied dd, 0 for actual dd. Affects IR and plots

% dividend growth shock.
if dd_option == 1; % use implied  
    xtdd = zeros(h,5); 
    e = 1; %rhomat(1,1); % one sd dp shock
    xtdd(2,:) = [e 0 e 0 0 ]; 
    for i = 3:h;
        xtdd(i,:) = xtdd(i-1,:)*A';
    end; 
end
if dd_option == 0; 
    xtdd = zeros(h,5); 
    e = 1; %rhomat(1,1); % one sd dp shock
    xtdd(2,:) = [e e 0 0 0 ]; 
    for i = 3:h;
        xtdd(i,:) = xtdd(i-1,:)*A';
    end; 
end

% dp shock. 
xtdp = zeros(h,5); 
e = 1; %= rhomat(3,3); % one sd dp shock
xtdp(2,:) = [-rho*e 0 0 e 0]; 
for i = 3:h;
    xtdp(i,:) = xtdp(i-1,:)*A';
end; 

% cay shock. 
xtcy = zeros(h,5); 
e = 1; %rhomat(4,4); % one sd cay shock
xtcy(2,:) = [0 0 0 0 e]; % no change to current dp or dd means no change to r 
for i = 3:h;
    xtcy(i,:) = xtcy(i-1,:)*A';
end; 

fprintf('Long-run impulse response \n');
sumrhoj = sum(  ((rho.^(0:h-3)')*(ones(1,5)) ).*xtcy(3:end,:));
fprintf('sum of rho^j-1 rt+j, dd t+j, dd_identity t+j  \n'); 
fprintf(' %10.3f %10.3f %10.3f \n', sumrhoj(1:3)); 

sumrhojdp = sum(  ((rho.^(0:h-3)')*(ones(1,5)) ).*xtdp(3:end,:));
fprintf('sum of rho^j-1 rt+j  dd t+j and dd_identity_t+j  for dp \n'); 
fprintf(' %10.3f %10.3f %10.3f \n', sumrhojdp(1:3)); 



if 1 % new graph with levels; requires ddoption to use identity dividends
    figure; % impulse response function
    subplot(2,3,1);
    plot((0:h-1)',xtdd(:,1),'-v','linewidth',2,'markersize',3);
    hold on;
    plot((0:h-1)',xtdd(:,3),'--r','linewidth',2,'markersize',3);
    %        plot((0:h-1)',cumsum(xtdd(:,3)),'-rv','linewidth',2,'markersize',3);
    plot([1 1],[-0.1 1.1],'--k','linewidth',2);
    plot((0:h-1)',0*(1:h)','-k')
    legend('Return','Div growth','Shock date');
    title('Response to \Deltad shock');
    xlabel('Years')
    axis([ 0 20 -0.1 1.1])
    
    subplot(2,3,2);
    plot((0:h-1)',xtdp(:,1),'-v','linewidth',2,'markersize',3);
    hold on;
    plot((0:h-1)',xtdp(:,3),'--r','linewidth',2,'markersize',3);
    plot((0:h-1)',0*(1:h)','-k')
    legend('Return','Div growth');
    title('Response to dp shock');
    axis([ 0 20 -0.05 0.15])
    text(10,0.1,['\Sigma\rho^{j-1}r_{t+j}=' num2str(sumrhojdp(1),'%4.2f')],'color','b')
    text(10,0.03,['\Sigma\rho^{j-1}\Deltad_{t+j}=' num2str(sumrhojdp(3),'%4.2f')],'color','r')
    [figx figy ] = dsxy2figxy([3.5 2.5],[-0.02 -0.045]);
    txtar =  annotation('arrow',figx,figy,'color','b');
    text(4,-0.02,'r_{t} = -0.96','color','b');
    plot([1 1],[-0.05 0.15],'--k','linewidth',2)
    xlabel('Years')
    
    subplot(2,3,3)
    plot((0:h-1)',xtcy(:,1),'-v','linewidth',2,'markersize',3);
    hold on;
    plot((0:h-1)',xtcy(:,3),'--r','linewidth',2,'markersize',3);
    plot((0:h-1)',0*(1:h)','-k')
    legend('Return','Div growth');
    title('Response to 1 \sigma cay shock');
    axis([ 0 20 -0.012 0.075])
    text(5,0.04,['\Sigma\rho^{j-1}r_{t+j}=' num2str(sumrhoj(1),'%4.3f')],'color','b')
    text(10,0.003,['\Sigma\rho^{j-1}\Deltad_{t+j}=' num2str(sumrhoj(3),'%4.3f')],'color','r')
    plot([1 1],[-0.012 0.075],'--k','linewidth',2)
    xlabel('Years')
  
    % delta p1 = -dp1 + dp0 + dd1;
    xtdeltap_dd = -xtdd(:,4) + [0;xtdd(1:end-1,4)] + xtdd(:,3);
    subplot(2,3,4);
    plot((0:h-1)',cumsum(xtdeltap_dd),'-v','linewidth',2,'markersize',3);
    hold on;
    plot((0:h-1)',cumsum(xtdd(:,3)),'--r','linewidth',2,'markersize',3);
    plot([1 1],[-0.1 1.1],'--k','linewidth',2);
    plot((0:h-1)',0*(1:h)','-k')
    legend('Price','Dividend','Shock date',4);
    title('Response to \Deltad shock');
    axis([ 0 20 -0.1 1.1])
    xlabel('Years')
    
    xtdeltap_dp = -xtdp(:,4) + [0;xtdp(1:end-1,4)] + xtdp(:,3);
    subplot(2,3,5);
    plot((0:h-1)',cumsum(xtdeltap_dp),'-v','linewidth',2,'markersize',3);
    hold on;
    plot((0:h-1)',cumsum(xtdp(:,3)),'--r','linewidth',2,'markersize',3);
    plot((0:h-1)',0*(1:h)','-k')
    legend('Price','Dividend',4);
    title('Response to dp shock');
    axis([ 0 20 -1.1 0.4])
    %text(10,0.1,['\Sigma\rho^{j-1}r_{t+j}=' num2str(sumrhojdp(1),'%4.2f')],'color','b')
    %text(10,0.03,['\Sigma\rho^{j-1}\Deltad_{t+j}=' num2str(sumrhojdp(3),'%4.2f')],'color','r')
    %[figx figy ] = dsxy2figxy([3.5 2.5],[-0.02 -0.045]);
    %txtar =  annotation('arrow',figx,figy,'color','b');
    %text(4,-0.02,'r_{t} = -0.96','color','b');
    plot([1 1],[-1.1 0.4],'--k','linewidth',2)
    xlabel('Years')
    
    xtdeltap_cay = -xtcy(:,4) + [0;xtcy(1:end-1,4)] + xtcy(:,3);
    subplot(2,3,6)
    plot((0:h-1)',cumsum(xtdeltap_cay),'-v','linewidth',2,'markersize',3);
    hold on;
    plot((0:h-1)',cumsum(xtcy(:,3)),'--r','linewidth',2,'markersize',3);
    plot((0:h-1)',0*(1:h)','-k')
    legend('price','dividend');
    title('Response to 1 \sigma cay shock');
    axis([ 0 20 -0.01 0.16])
    %text(5,0.04,['\Sigma\rho^{j-1}r_{t+j}=' num2str(sumrhoj(1),'%4.3f')],'color','b')
    %text(10,0.003,['\Sigma\rho^{j-1}\Deltad_{t+j}=' num2str(sumrhoj(3),'%4.3f')],'color','r')
    plot([1 1],[-0.01 0.16],'--k','linewidth',2)
    xlabel('Years')    
    
    set(gcf,'paperposition',[0.25 2.5 12 7.5])
    print -depsc2 cay_resp_dp_and_cay.eps;
end

if 0; % old picture
    figure; % impulse response function
    %subplot(2,2,1);
    %plot((1:h)',xtdd(:,1:2));
    %legend('return','div growth');
    %title('Response to \Deltad shock');
    
    div_option = 1;
    if div_option;
        subplot(1,3,1);
    else
        subplot(1,2,1)
    end;
    if div_option
        plot((0:h-1)',xtdd(:,1),'-v','linewidth',2,'markersize',2);
        hold on;
        if dd_option == 0;
            plot((0:h-1)',xtdd(:,2),'--ro','linewidth',2,'markersize',2);
            plot((0:h-1)',cumsum(xtdd(:,2)),'-rv','linewidth',2,'markersize',2);
        end;
        if dd_option == 1;
            plot((0:h-1)',xtdd(:,3),'--ro','linewidth',2,'markersize',2);
            plot((0:h-1)',cumsum(xtdd(:,3)),'-rv','linewidth',2,'markersize',2);
        end;
        %plot((0:h-1)',0*(1:h)','-k') overwrites returns
        plot([1 1],[0 1],'--k','linewidth',2);
        legend('return','div growth','dividends','shock date');
        title('Response to \Deltad shock');
        axis([ 0 20 -inf inf])
        %text(10,0.1,['\Sigma\rho^{j-1}r_{t+j}=' num2str(sumrhojdp(1),'%4.2f')],'color','b')
        %if dd_option == 0;
        %    text(10,0.03,['\Sigma\rho^{j-1}\Deltad_{t+j}=' num2str(sumrhojdp(2),'%4.2f')],'color','r')
        %end
        %if dd_option == 1;
        %    text(10,0.03,['\Sigma\rho^{j-1}\Deltad_{t+j}=' num2str(sumrhojdp(3),'%4.2f')],'color','r')
        %end
        %text(3,-0.04,'r_{t+1} = -0.96','color','b');
    end
    
    if div_option
        subplot(1,3,2);
    else
        subplot(1,2,1)
    end
    plot((0:h-1)',xtdp(:,1),'-v','linewidth',2,'markersize',2);
    hold on;
    if dd_option == 0;
        plot((0:h-1)',xtdp(:,2),'-ro','linewidth',2,'markersize',2);
    end;
    if dd_option == 1;
        plot((0:h-1)',xtdp(:,3),'-ro','linewidth',2,'markersize',2);
    end;
    plot((0:h-1)',0*(1:h)','-k')
    legend('return','div growth');
    title('Response to dp shock');
    axis([ 0 20 -0.05 0.15])
    text(10,0.1,['\Sigma\rho^{j-1}r_{t+j}=' num2str(sumrhojdp(1),'%4.2f')],'color','b')
    if dd_option == 0;
        text(10,0.03,['\Sigma\rho^{j-1}\Deltad_{t+j}=' num2str(sumrhojdp(2),'%4.2f')],'color','r')
    end
    if dd_option == 1;
        text(10,0.03,['\Sigma\rho^{j-1}\Deltad_{t+j}=' num2str(sumrhojdp(3),'%4.2f')],'color','r')
    end
    %text(3,-0.04,'r_{t+1} = -0.96','color','b');
    [figx figy ] = dsxy2figxy([3.5 2.5],[-0.02 -0.045]);
    txtar =  annotation('arrow',figx,figy,'color','b');
    text(4,-0.02,'r_{t} = -0.96','color','b');
    plot([1 1],[-0.05 0.15],'--k','linewidth',2)
    
    if div_option
        subplot(1,3,3)
    else
        subplot(1,2,2);
    end
    plot((0:h-1)',xtcy(:,1),'-v','linewidth',2,'markersize',2);
    hold on;
    if dd_option == 0;
        plot((0:h-1)',xtcy(:,2),'-ro','linewidth',2,'markersize',2);
    end;
    if dd_option == 1;
        plot((0:h-1)',xtcy(:,3),'-ro','linewidth',2,'markersize',2);
    end;
    plot((0:h-1)',0*(1:h)','-k')
    legend('return','div growth');
    title('Response to 1 \sigma cay shock');
    axis([ 0 20 -inf inf])
    text(5,0.04,['\Sigma\rho^{j-1}r_{t+j}=' num2str(sumrhoj(1),'%4.3f')],'color','b')
    if dd_option == 0;
        text(10,0.003,['\Sigma\rho^{j-1}\Deltad_{t+j}=' num2str(sumrhoj(2),'%4.3f')],'color','r')
    end
    if dd_option == 1;
        text(10,0.003,['\Sigma\rho^{j-1}\Deltad_{t+j}=' num2str(sumrhoj(3),'%4.3f')],'color','r')
    end
    plot([1 1],[-0.01 0.07],'--k','linewidth',2)
    
    set(gcf,'paperposition',[0.25 2.5 8 3])
    print -depsc2 cay_resp_dp_and_cay.eps;
end
% variance analysis - to be used in Section 2.4

varx = cov(err);
An = A; 
for i = 1:10; 
    varx = varx + An*varx*An';
    An = An*An;
end; % doubling algorithm computes very fast. 

lrvar = inv(eye(5)-rho*A)*A*varx*A'*(inv(eye(5)-rho*A))'; 
er = [1 0 0 0 0]'; 
ed = [0 1 0 0 0]';
edid = [0 0 1 0 0]';

fprintf('decomposition of variance with dp and cay, raw and in percent -- regular dd\n'); 
fprintf('                  %10.9s %10.9s %10.9s\n','var','sd','pct');
fprintf(' var(dp)          %10.3f %10.3f %10.3f\n',varx(4,4),varx(4,4)^0.5,varx(4,4)/varx(4,4)*100); 
fprintf(' var(Erlr)        %10.3f %10.3f %10.3f\n',er'*lrvar*er,(er'*lrvar*er)^0.5,er'*lrvar*er/varx(4,4)*100); 
fprintf(' var(Edlr)        %10.3f %10.3f %10.3f\n',ed'*lrvar*ed,(ed'*lrvar*ed)^0.5,ed'*lrvar*ed/varx(4,4)*100);
fprintf(' -2cov(Erlr,Edlr) %10.3f %10.3s %10.3f \n',-2*er'*lrvar*ed,' ',-2*er'*lrvar*ed/varx(4,4)*100); 
fprintf(' corr(Erlr,Edlr ) %10.3f  \n',er'*lrvar*ed/((er'*lrvar*er * ed'*lrvar*ed)^0.5)); 
fprintf(' sum              %10.3f %10.3f \n',er'*lrvar*er+ed'*lrvar*ed-2*er'*lrvar*ed,...
    (er'*lrvar*er+ed'*lrvar*ed-2*er'*lrvar*ed)/varx(4,4)*100); 

fprintf('decomposition of variance with dp and cay, raw and in percent -- dd from identity \n'); 
fprintf('                  %10.9s %10.9s %10.9s\n','var','sd','pct');
fprintf(' var(dp)          %10.3f %10.3f %10.3f\n',varx(4,4),varx(4,4)^0.5,varx(4,4)/varx(4,4)*100); 
fprintf(' var(Erlr)        %10.3f %10.3f %10.3f\n',er'*lrvar*er,(er'*lrvar*er)^0.5,er'*lrvar*er/varx(4,4)*100); 
fprintf(' var(Edidlr)      %10.3f %10.3f %10.3f\n',edid'*lrvar*edid,(edid'*lrvar*edid)^0.5,edid'*lrvar*edid/varx(4,4)*100);
fprintf('-2cov(Erlr,Edidlr)%10.3f %10.3s %10.3f \n',-2*er'*lrvar*edid,' ',-2*er'*lrvar*edid/varx(4,4)*100); 
fprintf(' corr(Erlr,Edidlr ) %10.3f  \n',er'*lrvar*edid/((er'*lrvar*er * edid'*lrvar*edid)^0.5)); 
fprintf(' sum, id          %10.3f %10.3f \n',er'*lrvar*er+edid'*lrvar*edid-2*er'*lrvar*edid,...
    (er'*lrvar*er+ed'*lrvar*edid-2*er'*lrvar*edid)/varx(4,4)*100); 


disp('note VAR produces different variances than sample averages.')
disp('table of standard deviations')
fprintf('        %10.9s  %10.9s  %10.9s  %10.9s  %10.9s  \n','r','dd','ddid','dp','cay');
fprintf(' sample %10.3f  %10.3f  %10.3f  %10.3f  %10.3f  \n',std(r),std(dd(2:end)),std(dd_id(2:end)),std(dp),std(cay_a) );
fprintf(' VAR    %10.3f  %10.3f  %10.3f  %10.3f  %10.3f  \n',diag(varx)'.^0.5);



% -----------------------------------------------
% CP do it better than goyal -welch (they did something wrong) 

x = load('data\famabliss.txt'); % quarterly updates

fbdate = x(:,1);
fbst = 19631231; % one more month so you have the year in annual regression 
%fbst=19640131;% CP beginning
beg = find(x(:,1)==fbst); % one more month so you have the year in annual regression
fblast = 20031231; % cp sample
%fblast = 20071231; % stop at crisis
last = find(x(:,1)==fblast);
p = log(x(:,2:end)/100);
T = size(x,1);
y = -p.*(ones(T,1)*[1 1/2 1/3 1/4 1/5]);
f = [y(:,1) p(:,1:end-1)-p(:,2:end)];
fma = (f(1:end-2,:)+f(2:end-1,:)+f(3:end,:))/3;
fma = [ ones(2,5)*NaN ; fma]; 
rb = [NaN*ones(12,4); [p(13:end,1:end-1)]-[p(1:end-12,2:end)]]; 
rbx = rb - [NaN(12,1); y(1:end-12,1)]*ones(1,4);

% see check_CP for many variations

rhv = [ones(size(fma(beg:last-12,:),1),1) fma(beg:last-12,:)]; 
lhv = mean(rbx(beg+12:last,:),2); 
[bv,sebv,R2v,R2vadj,v,F] = olsgmm(lhv,rhv,0,0);
disp('')
fprintf('CP regressions sample %16i %16i\n',fbst, fblast) 
fprintf('average returns on f moving average \n'); 
fprintf(' %6.2f %6.2f %6.2f %6.2f %6.2f %6.3f \n', bv(2:end,1), R2v(1));

cp = rhv*bv; 
cp_ann = cp(1:12:end); % first obs is 1963=19631231
r_cp = r((vw_dates>=fbst)&(vw_dates<=fblast));
rbx_ann = mean(rbx(beg+12:12:last,:),2);

rhv = [ones(size(cp_ann,1),1) cp_ann]; 
lhv = r_cp(2:end); 
[bv,sebv,R2v,R2vadj,v,F] = olsgmm(lhv,rhv,0,0);
fprintf('stock returns on cp factor, annual  \n'); 
fprintf('  %6.2f   %6.2f    %6.3f \n', bv, R2v(1));
fprintf(' (%6.2f) (%6.2f)         \n', bv./sebv);

lhv = rbx_ann; 
[bv,sebv,R2v,R2vadj,v,F] = olsgmm(lhv,rhv,0,0);
fprintf('bond average returns on cp factor, annual  \n'); 
fprintf('  %6.2f   %6.2f    %6.3f \n', bv, R2v(1));
fprintf(' (%6.2f) (%6.2f)         \n', bv./sebv);

%From macro.m in piazzesi: stock return forecasts seem not to work
%december/december. 
%vw excess returns on gpf
%    1.5660    1.8353    0.0637
%
%vw excess returns on gpf, annual
%    1.0115    1.1885    0.0355









            %% Section 2.5 Beyond Cay
%
' ************************ Beyond CAY *********************'

x = load('PredictorData2009_a.txt');

% yyyy	Index	D12	E12	b/m	tbl	AAA	BAA	lty	cay	ntis	Rfree	infl
% eqis	ltr	corpr	svar	csp	ik	CRSP_SPvw	CRSP_SPvwx Vix CP rv
% Bmspread Dpspread

t = x(:,1);
tt = find(and(t>=floor(Startdate/10000),t<=floor(Enddate/10000)));
x = x(tt,:);
t = x(:,1);
Index = x(:,2);  % S&P500 Index level
D12 = x(:,3);    % Dividends for S&P500 in year t
E12 = x(:,4);    % Earnings for S&P500 in year t
bm = x(:,5);     % DJIA book-to-market ratio
tbl = x(:,6);    % 3-month T-bill rate
AAA = x(:,7);    % Corporate bond yields with AAA ratings
BAA = x(:,8);    % Corporate bond yields with BAA ratings
lty = x(:,9);    % Long-term government bond yield (around 20yrs)
cay = x(:,10);   % Consumption-to-wealth ratio of Lettau-Ludvigson
ntis = x(:,11);  % Net stock issue of NYSE stocks, (Mcap_t-Mcap_{t-1}*r_t)/Mcap_t
Rfree = x(:,12); % Risk-free rate
infl = x(:,13);  % Inflation from CPI All Urban Consumer
eqis = x(:,14);  % Perceint Equity Issuing (fraction of equity issue among total issue)
ltr = x(:,15);   % Long-term government bond returns
corpr = x(:,16); % Long-term corporate bond returns
svar = x(:,17);  % Stock variance, sum of squared daily returns on S&P500
csp = x(:,18);   % Cross-sectional premium of Polk, Thompson and Vuolteenaho
ik = x(:,19);    % Investment-to-capital ratio of Cochrane
vix = x(:,22)/100;   % Vix
CP = x(:,23);    % Cochrane-Piazzezi Factor
rv = x(:,24);    % Monthly svar

ivrv = vix.^2 - rv;
dy = [log(D12(2:end))-log(Index(1:end-1)); NaN];
ep = log(E12)-log(Index);
de = log(D12)-log(E12);
tms = lty - tbl;
dfy = BAA - AAA;
dfr = corpr - ltr;

Pred = [bm cay ntis infl eqis svar csp ik dy ep de tms dfy dfr vix CP ivrv];
lbls = {'bm','cay','ntis','infl','eqis','svar','csp','ik','dy','ep','de','tms','dfy','dfr','vix','CP','ivrv'};
N = size(Pred,2);

for n = 1:N;
    %n = 2;   % Pick your favorite predictor
    disp([' ---------------------------------']);
    pred = Pred(:,n);
    
    tt = find(~isnan(pred));  % Chop data to match the length
    rs = r(tt);               % Here I use log return (not excess)
    preds = pred(tt);
    dps = dp(tt);
    dds = dd(tt);
    fprintf('Start %6.0f \n',t(tt(1)));
    fprintf('End   %6.0f \n',t(tt(end)));
    
    lhv = rs(2:end);    % Run univariate regression with the picked predictor
    rhv = [ones(length(tt)-1,1) preds(1:end-1)];
    [bv,sebv,R2v,R2vadj,v,F] = olsgmm(lhv,rhv,0,0);
    
    fprintf('Forecasting regression with %4s \n',char(lbls(n)));
    fprintf('           %6s %6s %6s \n','a','b','R2');
    fprintf('Estimate   %6.2f %6.2f %6.2f \n',bv(1),bv(2),R2v);
    fprintf('t-stats    %6.2f %6.2f \n',bv(1)/sebv(1),bv(2)/sebv(2));
    
    lhv = rs(2:end);  % Multivariate regression with log dp
    rhv = [ones(length(tt)-1,1) dps(1:end-1) preds(1:end-1)];
    [bv,sebv,R2v,R2vadj,v,F] = olsgmm(lhv,rhv,0,0);
    
    fprintf('Forecasting regression with dp and %4s \n',char(lbls(n)));
    fprintf('           %6s %6s %6s %6s %6s \n','a','b_dp','b_pred','R2','corr(pred,dp)');
    fprintf('Estimate   %6.2f %6.2f %6.2f %6.2f %6.2f \n',bv(1),bv(2),bv(3),R2v,corr(dps,preds));
    fprintf('t-stats    %6.2f %6.2f %6.2f \n',bv(1)/sebv(1),bv(2)/sebv(2),bv(3)/sebv(3));
    
    % VAR
    lhv = [rs(2:end) dds(2:end) dps(2:end) preds(2:end)];
    rhv = [ones(length(tt)-1,1) dps(1:end-1) preds(1:end-1)];
    [bv,sebv,R2v,R2vadj,v,F] = olsgmm(lhv,rhv,0,0);
    b = bv(2:3,:);
    A = [zeros(4,2) b'];    % expand to [r dd dp cay]' = A []'_t  VAR
    err = lhv - rhv*bv;
    
    Al = A*inv(eye(size(A))-rho*A);
    brl = Al(1,3:4);
    bdl = Al(2,3:4);
    
    fprintf('Long-run VAR coefficient of return on dp and %4s \n',char(lbls(n)));
    fprintf('           %6s %6s \n','b_dp','b_pred');
    fprintf('br-longrun %6.2f %6.2f \n',brl);
    disp('');
    
    ESumrhor = [dps(1:end-1) preds(1:end-1)]*Al(1,3:4)';
    ESumrhod = [dps(1:end-1) preds(1:end-1)]*Al(2,3:4)';
    
    disp('Variance Decomposition');
    fprintf('s(dp)        %6.3f \n',std(dps));
    fprintf('s(ESumrhor)  %6.3f \n',std(ESumrhor));
    fprintf('s(ESumrhod)  %6.3f \n',std(ESumrhod));
    
    % variance analysis - to be used in Section 2.4
    
    varx = cov(err);
    An = A;
    for i = 1:10;
        varx = varx + An*varx*An';
        An = An*An;
    end; % doubling algorithm computes very fast.
    
    lrvar = inv(eye(4)-rho*A)*A*varx*A'*(inv(eye(4)-rho*A))';
    er = [1 0 0 0]';
    ed = [0 1 0 0]';
    
    fprintf('decomposition of variance with dp and %4s \n',char(lbls(n)));
    fprintf('                  %10.9s %10.9s %10.9s\n','var','sd','pct');
    fprintf(' var(dp)          %10.3f %10.3f %10.3f\n',varx(3,3),varx(3,3)^0.5,varx(3,3)/varx(3,3)*100);
    fprintf(' var(Erlr)        %10.3f %10.3f %10.3f\n',er'*lrvar*er,(er'*lrvar*er)^0.5,er'*lrvar*er/varx(3,3)*100);
    fprintf(' var(Edlr)        %10.3f %10.3f %10.3f\n',ed'*lrvar*ed,(ed'*lrvar*ed)^0.5,ed'*lrvar*ed/varx(3,3)*100);
    fprintf(' -2cov(Erlr,Edlr) %10.3f %10.3s %10.3f \n',-2*er'*lrvar*ed,' ',-2*er'*lrvar*ed/varx(3,3)*100);
    fprintf(' corr(Erlr,Edlr ) %10.3f  \n',er'*lrvar*ed/((er'*lrvar*er * ed'*lrvar*ed)^0.5));
    fprintf(' sum              %10.3f %10.3f \n',er'*lrvar*er+ed'*lrvar*ed-2*er'*lrvar*ed,...
        (er'*lrvar*er+ed'*lrvar*ed-2*er'*lrvar*ed)/varx(3,3)*100);
    
    disp('note VAR produces different variances than sample averages.')
    disp('table of standard deviations')
    fprintf('        %10.9s  %10.9s  %10.9s  %10.9s  \n','r','dd','dp','cay');
    fprintf(' VAR    %10.3f  %10.3f  %10.3f  %10.3f  \n',std(r),std(dd(2:end)),std(dp),std(pred) );
    fprintf(' sample %10.3f  %10.3f  %10.3f  %10.3f  \n',diag(varx)'.^0.5);
    
end


             